<?php
namespace St\Plugin\Blocks\Gutenberg;

use St\Plugin\Blocks\Event_Listener_Interface;
use St\Plugin\Blocks\Plugin\Plugin_Meta;
use WP_Post;

/**
 * Class Gutenberg_Subscriber.
 *
 * @package St\Plugin\Blocks\Gutenberg
 */
class Gutenberg_Subscriber implements Event_Listener_Interface {
	/**
	 * @var Plugin_Meta
	 */
	private $plugin_meta;

	/**
	 * Gutenberg_Subscriber constructor.
	 *
	 * @param Plugin_Meta $plugin_meta
	 */
	public function __construct( Plugin_Meta $plugin_meta ) {
		$this->plugin_meta = $plugin_meta;
	}

	public function subscribe() {
		add_action( 'init', [ $this, 'on_init' ] );
		add_action( 'enqueue_block_editor_assets', [ $this, 'on_block_editor_assets' ] );
		add_filter( 'block_categories', [ $this, 'on_block_categories' ], 10, 2 );
	}

	public function on_init() {
		do_action( ST_BLOCKS_API_PREFIX . '_before_register_block_styles' );

		wp_register_style(
			'st-blocks',
			$this->plugin_meta->get_url() . 'assets/css/index.css',
			['wp-components'],
			$this->plugin_meta->get_version()
		);

		do_action( ST_BLOCKS_API_PREFIX . '_after_register_block_styles' );

		do_action( ST_BLOCKS_API_PREFIX . '_before_register_block_scripts' );

		wp_register_script(
			'st-blocks',
			$this->plugin_meta->get_url() . 'assets/js/index.js',
			[
				'wp-blocks',
				'wp-element',
				'wp-editor',
				'wp-components',
				'wp-rich-text',
				'wp-i18n',
			],
			$this->plugin_meta->get_version(),
			true
		);

		$st_kaiwa_shortcodes      = $this->generate_st_kaiwa_shortcode_data();
		$st_kaiwa_shortcodes_json = wp_json_encode( $st_kaiwa_shortcodes );

		$st_blocks_before = <<<JS
(function (window, undefined) {
	'use strict';

	var SHORTCODES = {$st_kaiwa_shortcodes_json}

	window.ST_BLOCKS                     = window.ST_BLOCKS || {};
	window.ST_BLOCKS.ST_KAIWA            = window.ST_BLOCKS.ST_KAIWA || {};
	window.ST_BLOCKS.ST_KAIWA.SHORTCODES = window.ST_BLOCKS.ST_KAIWA.SHORTCODES || SHORTCODES;
}(window));
JS;

		wp_add_inline_script( 'st-blocks', $st_blocks_before, 'before' );

		do_action( ST_BLOCKS_API_PREFIX . '_after_register_block_scripts' );
	}

	/**
	 * @param array<array<string, mixed>> $categories
	 * @param WP_Post $post
	 *
	 * @return array<array<string, mixed>>
	 */
	public function on_block_categories( array $categories, WP_Post $post ) {
		return array_merge(
			$categories,
			[
				[
					'slug'  => 'st-blocks',
					'title' => 'STINGER',
				],
			]
		);
	}

	public function on_block_editor_assets() {
		do_action( ST_BLOCKS_API_PREFIX . '_before_enqueue_block_styles' );
		wp_enqueue_style( 'st-blocks' );
		do_action( ST_BLOCKS_API_PREFIX . '_after_enqueue_block_styles' );

		do_action( ST_BLOCKS_API_PREFIX . '_before_enqueue_block_scripts' );
		wp_enqueue_script( 'st-blocks' );
		do_action( ST_BLOCKS_API_PREFIX . '_after_enqueue_block_scripts' );
	}

	/**
	 * @return array<int, array<string, array<string, mixed>>>
	 */
	private function generate_st_kaiwa_shortcode_data() {
		$normalize_shortcode_data = function ( $tag_name, array $data = [] ) {
			$label = isset( $data['label'] ) ? $data['label'] : '[' . $tag_name . ']';

			$defaults = [
				// For Gutenberg.
				'tagName'      => $tag_name,
				'label'        => $label,
				'classes'      => [],

				// For Shortcode.
				'iconName'     => null,
				'iconImageUrl' => null,
			];

			return array_merge( $defaults, $data );
		};

		// Shortcode 1.
		$shortcode_data_1 = apply_filters(
			ST_BLOCKS_API_PREFIX . '_st_kaiwa_shortcode_data_1',
			[
				'st-kaiwa-1' => $normalize_shortcode_data( 'st-kaiwa-1' ),
			]
		);

		foreach ( $shortcode_data_1 as $tag_name => $data ) {
			$shortcode_data_1[ $tag_name ] = $normalize_shortcode_data( $tag_name, $data );
		}

		// Shortcode 2.
		$shortcode_data_2 = apply_filters( ST_BLOCKS_API_PREFIX . '_st_kaiwa_shortcode_data_2', [] );

		foreach ( $shortcode_data_2 as $tag_name => $data ) {
			$shortcode_data_2[ $tag_name ] = $normalize_shortcode_data( $tag_name, $data );
		}

		return [ $shortcode_data_1, $shortcode_data_2 ];
	}
}
